/*
 * Copyright 2021 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

/***********************************************************************************/
/* This file is automatically generated using bindtool and can be manually edited  */
/* The following lines can be configured to regenerate this file during cmake      */
/* If manual edits are made, the following tags should be modified accordingly.    */
/* BINDTOOL_GEN_AUTOMATIC(0)                                                       */
/* BINDTOOL_USE_PYGCCXML(0)                                                        */
/* BINDTOOL_HEADER_FILE(fcdpp.h)                                                   */
/* BINDTOOL_HEADER_FILE_HASH(ede7534140bd9b838979fc32f9d259c6)                     */
/***********************************************************************************/

#include <pybind11/complex.h>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

namespace py = pybind11;

#include <funcube/fcdpp.h>
// pydoc.h is automatically generated in the build directory
#include <fcdpp_pydoc.h>

void bind_fcdpp(py::module& m)
{

    using fcdpp    = ::gr::funcube::fcdpp;


    py::class_<fcdpp, gr::hier_block2,
        std::shared_ptr<fcdpp>>(m, "fcdpp", D(fcdpp))

        .def(py::init(&fcdpp::make),
           py::arg("device_name") = "",
           py::arg("unit") = 1,
           D(fcdpp,make)
        )
        




        
        .def("set_freq",&fcdpp::set_freq,       
            py::arg("freq"),
            D(fcdpp,set_freq)
        )


        
        .def("set_lna",&fcdpp::set_lna,       
            py::arg("gain"),
            D(fcdpp,set_lna)
        )


        
        .def("set_mixer_gain",&fcdpp::set_mixer_gain,       
            py::arg("gain"),
            D(fcdpp,set_mixer_gain)
        )


        
        .def("set_freq_corr",&fcdpp::set_freq_corr,       
            py::arg("ppm"),
            D(fcdpp,set_freq_corr)
        )


        
        .def("set_if_gain",&fcdpp::set_if_gain,       
            py::arg("gain"),
            D(fcdpp,set_if_gain)
        )

        ;




}








