/* -*- c++ -*- */
/*
 * Copyright 2020 dl1ksv.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#include <funcube/fcdpp_control.h>
#include <gnuradio/attributes.h>
#include <boost/test/unit_test.hpp>

namespace gr {
  namespace funcube {

    BOOST_AUTO_TEST_CASE(test_fcdpp_control_replace_with_specific_test_name)
    {
      // Put test here
    }

  } /* namespace funcube */
} /* namespace gr */
